#include "mesh.h"
#include "meshbuilder.h"
#include "postprocessor.h"
#include "shader.h"

namespace {

const char fullscreen_vs_source[] =
	"attribute vec4 vertex;\n"
	"varying vec2 texcoord;\n"
	"void main()\n"
	"{\n"
	"	gl_Position = vertex;\n"
	"	texcoord = vertex.xy*0.5+0.5;\n"
	"}\n";

}


namespace Msp {
namespace GL {

void PostProcessor::render(Renderer &, const Texture2D &color, const Texture2D &depth)
{
	render(color, depth);
}

Shader &PostProcessor::get_fullscreen_vertex_shader()
{
	static VertexShader shader(fullscreen_vs_source);
	return shader;
}

const Mesh &PostProcessor::get_fullscreen_quad()
{
	static const Mesh &mesh = create_fullscreen_quad();
	return mesh;
}

const Mesh &PostProcessor::create_fullscreen_quad()
{
	static Mesh mesh(VERTEX2);
	MeshBuilder builder(mesh);
	builder.begin(TRIANGLE_STRIP);
	builder.vertex(-1, 1);
	builder.vertex(-1, -1);
	builder.vertex(1, 1);
	builder.vertex(1, -1);
	builder.end();
	return mesh;
}

} // namespace GL
} // namespace Msp
